Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var BufferSize = NumWorkers
BufferSize defines the size of the job buffer
var ErrBrokenSymlink = errors.New("broken symlink")
ErrBrokenSymlink is returned when a symlink is not pointing to as proper path
var NumWorkers = runtime.GOMAXPROCS(0)
NumWorkers defines how many workers to run on each Walk() function invocation
Functions ¶
Types ¶
type Walker ¶
type Walker struct {
// contains filtered or unexported fields
}
Walker is constructed for each Walk() function invocation
type WalkerError ¶
type WalkerError struct {
// contains filtered or unexported fields
}
WalkerError struct stores individual errors reported from each worker routine
func (WalkerError) Error ¶
func (we WalkerError) Error() string
Implement the error interface for WalkerError
type WalkerErrorList ¶
type WalkerErrorList struct {
ErrorList []WalkerError
}
WalkerErrorList struct store a list of errors reported from all worker routines
func (WalkerErrorList) Error ¶
func (wel WalkerErrorList) Error() string
Implement the error interface fo WalkerErrorList
Click to show internal directories.
Click to hide internal directories.